@@ -137,7 +137,6 @@ namespace System.Windows.Forms
 			MouseMove += new MouseEventHandler (OnMouseMoveLB);
 			MouseUp += new MouseEventHandler (OnMouseUpLB);
 			MouseWheel += new MouseEventHandler (OnMouseWheelLB);
-			KeyDown += new KeyEventHandler (OnKeyDownLB);
 			KeyUp += new KeyEventHandler (OnKeyUpLB);
 			GotFocus += new EventHandler (OnGotFocus);
 			LostFocus += new EventHandler (OnLostFocus);
@@ -1205,6 +1204,17 @@ namespace System.Windows.Forms
 
 		protected override void WndProc (ref Message m)
 		{
+			if ((Msg)m.Msg == Msg.WM_KEYDOWN) {
+				if (ProcessKeyMessage (ref m))
+					m.Result = IntPtr.Zero;
+				else {
+					HandleKeyDown ((Keys)m.WParam.ToInt32 ());
+					DefWndProc (ref m);
+				}
+
+				return;
+			}
+
 			base.WndProc (ref m);
 		}
 
@@ -1335,7 +1345,8 @@ namespace System.Windows.Forms
 					rect.Width += hscrollbar.Value;
 				}
 
-				OnDrawItem (new DrawItemEventArgs (dc, Font, rect, i, state, ForeColor, BackColor));
+				Color fore_color = (state & DrawItemState.Selected) != 0 ? ThemeEngine.Current.ColorHighlightText : ForeColor;
+				OnDrawItem (new DrawItemEventArgs (dc, Font, rect, i, state, fore_color, BackColor));
 			}
 		}
 
@@ -1375,8 +1386,14 @@ namespace System.Windows.Forms
 				if (hbar_offset < 0)
 					hbar_offset = 0;
 
-				if (IsHandleCreated)
+				if (IsHandleCreated) {
 					XplatUI.ScrollWindow (Handle, items_area, old_offset - hbar_offset, 0, false);
+
+					// Invalidate the previous selection border, to keep it properly updated.
+					Rectangle selection_border_area = new Rectangle (items_area.Width - (hbar_offset - old_offset) - 3, 0, 
+							3, items_area.Height);
+					Invalidate (selection_border_area);
+				}
 			}
 		}
 
@@ -1649,17 +1666,17 @@ namespace System.Windows.Forms
 			return true;
 		}
 
-		internal void OnKeyDownLB (object sender, KeyEventArgs e)
+		internal void HandleKeyDown (Keys key)
 		{
 			int new_item = -1;
 			
 			if (Items.Count == 0)
 				return;
 
-			if (KeySearch (e.KeyCode))
+			if (KeySearch (key))
 				return;
 
-			switch (e.KeyCode) {
+			switch (key) {
 				
 				case Keys.ControlKey:
 					ctrl_pressed = true;
@@ -2091,6 +2108,8 @@ namespace System.Windows.Forms
 			get {
 				if (string_format == null) {
 					string_format = new StringFormat ();
+					string_format.FormatFlags = StringFormatFlags.NoWrap;
+
 					if (RightToLeft == RightToLeft.Yes)
 						string_format.Alignment = StringAlignment.Far;
 					else
@@ -2196,7 +2215,7 @@ namespace System.Windows.Forms
 				show = true;
 				hscrollbar.Maximum = canvas_size.Width;
 				hscrollbar.LargeChange = Math.Max (0, items_area.Width);
-			} else if (horizontal_scrollbar) {
+			} else if (scroll_always_visible && horizontal_scrollbar) {
 				show = true;
 				enabled = false;
 				hscrollbar.Maximum = 0;
