@@ -154,12 +154,15 @@ namespace Microsoft.Build.Tasks {
 					continue;
 				}
 
-				Log.LogMessage (MessageImportance.Low, "Primary Reference {0}", item.ItemSpec);
+				LogWithPrecedingNewLine (MessageImportance.Low, "Primary Reference {0}", item.ItemSpec);
 				ResolvedReference resolved_ref = ResolveReference (item, searchPaths, true);
 				if (resolved_ref == null) {
 					Log.LogWarning ("Reference '{0}' not resolved", item.ItemSpec);
-					assembly_resolver.LogSearchLoggerMessages ();
+					assembly_resolver.LogSearchLoggerMessages (MessageImportance.Normal);
 				} else {
+					if (Environment.GetEnvironmentVariable ("XBUILD_LOG_REFERENCE_RESOLVER") != null)
+						assembly_resolver.LogSearchLoggerMessages (MessageImportance.Low);
+
 					Log.LogMessage (MessageImportance.Low,
 							"\tReference {0} resolved to {1}. CopyLocal = {2}",
 							item.ItemSpec, resolved_ref.TaskItem,
@@ -221,7 +224,8 @@ namespace Microsoft.Build.Tasks {
 				} else {
 					resolved = assembly_resolver.FindInDirectory (
 							item, spath,
-							allowedAssemblyExtensions ?? default_assembly_extensions);
+							allowedAssemblyExtensions ?? default_assembly_extensions,
+							specific_version);
 				}
 
 				if (resolved != null)
@@ -239,10 +243,13 @@ namespace Microsoft.Build.Tasks {
 			specific_version = true;
 			string value = item.GetMetadata ("SpecificVersion");
 			if (String.IsNullOrEmpty (value)) {
-				AssemblyName name = new AssemblyName (item.ItemSpec);
+				//AssemblyName name = new AssemblyName (item.ItemSpec);
 				// If SpecificVersion is not specified, then
 				// it is true if the Include is a strong name else false
-				specific_version = assembly_resolver.IsStrongNamed (name);
+				//specific_version = assembly_resolver.IsStrongNamed (name);
+
+				// msbuild seems to just look for a ',' in the name :/
+				specific_version = item.ItemSpec.IndexOf (',') >= 0;
 				return true;
 			}
 
@@ -264,22 +271,25 @@ namespace Microsoft.Build.Tasks {
 				assembly_resolver.ResetSearchLogger ();
 
 				if (!File.Exists (item.ItemSpec)) {
-					Log.LogMessage (MessageImportance.Low,
+					LogWithPrecedingNewLine (MessageImportance.Low,
 							"Primary Reference from AssemblyFiles {0}, file not found. Ignoring",
 							item.ItemSpec);
 					continue;
 				}
 
-				Log.LogMessage (MessageImportance.Low, "Primary Reference from AssemblyFiles {0}", item.ItemSpec);
+				LogWithPrecedingNewLine (MessageImportance.Low, "Primary Reference from AssemblyFiles {0}", item.ItemSpec);
 				string copy_local;
 
 				AssemblyName aname = assembly_resolver.GetAssemblyNameFromFile (item.ItemSpec);
 				if (aname == null) {
 					Log.LogWarning ("Reference '{0}' not resolved", item.ItemSpec);
-					assembly_resolver.LogSearchLoggerMessages ();
+					assembly_resolver.LogSearchLoggerMessages (MessageImportance.Normal);
 					continue;
 				}
 
+				if (Environment.GetEnvironmentVariable ("XBUILD_LOG_REFERENCE_RESOLVER") != null)
+					assembly_resolver.LogSearchLoggerMessages (MessageImportance.Low);
+
 				ResolvedReference rr = assembly_resolver.GetResolvedReference (item, item.ItemSpec, aname, true,
 						SearchPath.RawFileName);
 				copy_local = rr.CopyLocal.ToString ();
@@ -343,7 +353,7 @@ namespace Microsoft.Build.Tasks {
 			if (TryGetResolvedReferenceByAssemblyName (aname, false, out resolved_ref))
 				return resolved_ref;
 
-			Log.LogMessage (MessageImportance.Low, "Dependency {0}", aname);
+			LogWithPrecedingNewLine (MessageImportance.Low, "Dependency {0}", aname);
 			Log.LogMessage (MessageImportance.Low, "\tRequired by {0}", parent_asm_name);
 
 			ITaskItem item = new TaskItem (aname.FullName);
@@ -351,6 +361,9 @@ namespace Microsoft.Build.Tasks {
 			resolved_ref = ResolveReference (item, dependency_search_paths, false);
 
 			if (resolved_ref != null) {
+				if (Environment.GetEnvironmentVariable ("XBUILD_LOG_REFERENCE_RESOLVER") != null)
+						assembly_resolver.LogSearchLoggerMessages (MessageImportance.Low);
+
 				Log.LogMessage (MessageImportance.Low, "\tReference {0} resolved to {1}.",
 					aname, resolved_ref.TaskItem.ItemSpec);
 
@@ -380,7 +393,7 @@ namespace Microsoft.Build.Tasks {
 				}
 			} else {
 				Log.LogWarning ("Reference '{0}' not resolved", aname);
-				assembly_resolver.LogSearchLoggerMessages ();
+				assembly_resolver.LogSearchLoggerMessages (MessageImportance.Normal);
 			}
 
 			return resolved_ref;
@@ -460,7 +473,7 @@ namespace Microsoft.Build.Tasks {
 				return false;
 
 			// match for full name
-			if (AssemblyResolver.AssemblyNamesCompatible (key_aname, found_ref.AssemblyName, true))
+			if (AssemblyResolver.AssemblyNamesCompatible (key_aname, found_ref.AssemblyName, true, false))
 				// exact match, so its already there, dont add anything
 				return true;
 
@@ -487,6 +500,12 @@ namespace Microsoft.Build.Tasks {
 			return true;
 		}
 
+		void LogWithPrecedingNewLine (MessageImportance importance, string format, params object [] args)
+		{
+			Log.LogMessage (importance, String.Empty);
+			Log.LogMessage (importance, format, args);
+		}
+
 		// conflict b/w @main and @conflicting, picking @main
 		void LogConflictWarning (string main, string conflicting)
 		{
